/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RealCAENPsdDriver.h"

#include <controllers/common/acquisition/mode/DPPAcquisitionCommon.h>
#include "CAENPsdDriver.h"

using namespace std;

namespace caen_psd {

/*
 * Constructor
 */
RealCAENPsdDriver::RealCAENPsdDriver(CAENPsdDriver* owner) : CAENPsdState(owner) {

	m_boardHandle = 0;
	ret = CAEN_DGTZ_Success;
}

/*
 * Destructor
 */
RealCAENPsdDriver::~RealCAENPsdDriver() {

}

/*!
 * \brief open command implementation
 */
void RealCAENPsdDriver::open() {

	boardParams.LinkType = (CAEN_DGTZ_ConnectionType)owner()->linkType();
	boardParams.VMEBaseAddress = owner()->address();
	ret = CAEN_DGTZ_OpenDigitizer(boardParams.LinkType, 0, 0, boardParams.VMEBaseAddress, &m_boardHandle);
	if (ret) {
		cerr << "Error CAEN_DGTZ_OpenDigitizer: " << ret << endl;
		return;
	}
}

/*!
 * \brief close command implementation
 */
void RealCAENPsdDriver::close() {

	CAEN_DGTZ_CloseDigitizer(m_boardHandle);
}

/*!
 * \brief Init command implementation
 */
void RealCAENPsdDriver::init() {

	reset();

	readInfos();

	open();

    // Channel CTRL register (individual trigger, sequencial readout)
	ret = CAEN_DGTZ_WriteRegister(m_boardHandle, 0x8000, 0x01000114);
	if (ret) {
		cerr << "Error CAEN_DGTZ_WriteRegister: " << ret << endl;
		close();
		return;
	}


	// Set acquisition mode
	if (owner()->dppAcqMode() == acquisition::DPPAcquisitionCommon::LISTMODE_MODE) {
		boardParams.DPPAcqMode = CAEN_DGTZ_DPP_ACQ_MODE_List;
	}
	else if (owner()->dppAcqMode() == acquisition::DPPAcquisitionCommon::SCOPE_MODE) {
		boardParams.DPPAcqMode = CAEN_DGTZ_DPP_ACQ_MODE_Oscilloscope;
	}
	else if (owner()->dppAcqMode() == acquisition::DPPAcquisitionCommon::WAVEFORM_MODE) {
		boardParams.DPPAcqMode = CAEN_DGTZ_DPP_ACQ_MODE_Oscilloscope;
	}

	boardParams.SaveMode = (CAEN_DGTZ_DPP_SaveParam_t)owner()->saveMode();
	ret = CAEN_DGTZ_SetDPPAcquisitionMode(m_boardHandle, boardParams.DPPAcqMode, boardParams.SaveMode);
	if (ret) {
		cerr << "Error CAEN_DGTZ_SetDPPAcquisitionMode: " << ret << endl;
		close();
		return;
	}

	// Set start acquisition mode
	boardParams.AcqMode = (CAEN_DGTZ_AcqMode_t)owner()->acqMode();
	ret = CAEN_DGTZ_SetAcquisitionMode(m_boardHandle, CAEN_DGTZ_S_IN_CONTROLLED); //boardParams.AcqMode);
	if (ret) {
		cerr << "Error CAEN_DGTZ_SetAcquisitionMode: " << ret << endl;
		close();
		return;
	}

	// Set I/O level
	boardParams.IOLevel = (CAEN_DGTZ_IOLevel_t)owner()->ioLevel();
	ret = CAEN_DGTZ_SetIOLevel(m_boardHandle, boardParams.IOLevel);
	if (ret) {
		cerr << "Error CAEN_DGTZ_SetIOLevel: " << ret << endl;
		close();
		return;
	}

	// Set external trigger
	boardParams.TriggerMode = (CAEN_DGTZ_TriggerMode_t)owner()->triggerMode();
	ret = CAEN_DGTZ_SetExtTriggerInputMode(m_boardHandle, boardParams.TriggerMode);
	if (ret) {
		cerr << "Error CAEN_DGTZ_SetExtTriggerInputMode: " << ret << endl;
		close();
		return;
	}

	// Enable channels
	boardParams.ChannelMask = owner()->channelMask();
	ret = CAEN_DGTZ_SetChannelEnableMask(m_boardHandle, boardParams.ChannelMask);
	if (ret) {
		cerr << "Error CAEN_DGTZ_SetChannelEnableMask: " << ret << endl;
		close();
		return;
	}

	// Set aggregate size - how many events per output buffer
	// 0 = automatic
	if (owner()->dppAcqMode() == acquisition::DPPAcquisitionCommon::LISTMODE_MODE) {
		boardParams.EventAggr = 0; //2
	}
	else if (owner()->dppAcqMode() == acquisition::DPPAcquisitionCommon::SCOPE_MODE) {
		boardParams.EventAggr = 1; //10
	}
	else if (owner()->dppAcqMode() == acquisition::DPPAcquisitionCommon::WAVEFORM_MODE) {
			boardParams.EventAggr = 1; //10
	}
	else {
		boardParams.EventAggr = 0;
	}

	ret = CAEN_DGTZ_SetDPPEventAggregation(m_boardHandle, boardParams.EventAggr, 0);
	if (ret) {
		cout << "1751 error: " << boardParams.EventAggr << endl;
		cerr << "Error CAEN_DGTZ_SetDPPEventAggregation: " << ret << endl;
		close();
		return;
	}

	// Set multi-board synchronization mode
	ret = CAEN_DGTZ_SetRunSynchronizationMode(m_boardHandle, CAEN_DGTZ_RUN_SYNC_Disabled);
	if (ret) {
		cerr << "Error CAEN_DGTZ_SetRunSynchronizationMode: " << ret << endl;
		close();
		return;
	}

	// Set virtual probe settings
	ret = CAEN_DGTZ_SetDPP_PSD_VirtualProbe(m_boardHandle, CAEN_DGTZ_DPP_VIRTUALPROBE_SINGLE,
			CAEN_DGTZ_DPP_PSD_VIRTUALPROBE_Baseline, CAEN_DGTZ_DPP_PSD_DIGITALPROBE1_R6_GateLong,
			CAEN_DGTZ_DPP_PSD_DIGITALPROBE2_R6_GateShort);
	if (ret) {
		cerr << "Error CAEN_DGTZ_SetDPP_VirtualProbe: " << ret << endl;
		close();
		return;
	}

	uint32 data;
	ret = CAEN_DGTZ_ReadRegister(m_boardHandle, 0x8000, &data);

	if (owner()->dppAcqMode() == acquisition::DPPAcquisitionCommon::LISTMODE_MODE) {
		data = 0x000E0110;
	}
	else if (owner()->dppAcqMode() == acquisition::DPPAcquisitionCommon::SCOPE_MODE) {
		//data = 0x01870110;
		data = 0x018F0110;
	}
	else if (owner()->dppAcqMode() == acquisition::DPPAcquisitionCommon::WAVEFORM_MODE) {
		data = 0x018F0110;
	}
	ret = CAEN_DGTZ_WriteRegister(m_boardHandle, 0x8000, data);
	if (ret) {
		cerr << "Error CAEN_DGTZ_WriteRegister: " << ret << endl;
		close();
		return;
	}
	ret = CAEN_DGTZ_WriteRegister(m_boardHandle, 0x8100, 0x5);
	if (ret) {
		cerr << "Error CAEN_DGTZ_WriteRegister: " << ret << endl;
		close();
		return;
	}

	close();

	//readInfos();

	writeParam();
}

/*!
 * \brief Clear command implementation
 */
void RealCAENPsdDriver::clear() {

	open();
	ret = CAEN_DGTZ_ClearData(m_boardHandle);
	if (ret) {
		cerr << "Error CAEN_DGTZ_ClearData: " << ret << endl;
	}
	close();
}

/*!
 * \brief Reset command implementation
 */
void RealCAENPsdDriver::reset() {

	open();
	ret = CAEN_DGTZ_Reset(m_boardHandle);
	if (ret) {
		cerr << "Error CAEN_DGTZ_Reset: " << ret << endl;
	}
	close();
}

/*!
 * \brief Write Param command implementation
 */
void RealCAENPsdDriver::writeParam() {

	cout << owner()->getName() << " driver set acq mode " << owner()->dppAcqMode() << endl;
	// Load DPP parameters into structure
	for (int32 i=0; i<owner()->nbChannels(); ++i) {
		dppParams.trgho = owner()->triggerHoldOff();
		dppParams.thr[i] = owner()->triggerThreshold.get(i);
		dppParams.selft[i] = owner()->selfTrigger.get(i);
		dppParams.csens[i] = owner()->chargeSensitivity.get(i);
		dppParams.sgate[i] = owner()->shortGate.get(i);
		dppParams.lgate[i] = owner()->longGate.get(i);
		dppParams.pgate[i] = owner()->preGate.get(i);
		dppParams.tvaw[i] = owner()->triggerValidationWindow.get(i);
		dppParams.nsbl[i] = owner()->baselineMean.get(i);
	}

	open();

    // Write channels parameters
	ret = CAEN_DGTZ_SetDPPParameters(m_boardHandle, boardParams.ChannelMask, &dppParams);
	if (ret) {
		cerr << "Error CAEN_DGTZ_SetDPPParameters" << endl;
		close();
		return;
	}

	for (int32 i=0; i<owner()->channels(); ++i) {
		if (owner()->channelMask() & (1 << i)) {
			// Set nb of samples for oscilloscope mode (same for all channels)
			boardParams.RecordLength = owner()->recordLength();
			ret = CAEN_DGTZ_SetRecordLength(m_boardHandle, boardParams.RecordLength, i);
			if (ret) {
				cerr << "Error CAEN_DGTZ_SetRecordLength: " << ret << endl;
				close();
				return;
			}

			// Set DC offset
			int32 offset = owner()->dcOffset.get(i);
			offset +=50;
			offset = offset * 65535 / 100;
			ret = CAEN_DGTZ_SetChannelDCOffset(m_boardHandle, i, offset);
			if (ret) {
				cerr << "Error CAEN_DGTZ_SetChannelDCOffset" << endl;
				close();
				return;
			}

			// Set Pre-trigger size
			int32 ptgrSize = owner()->preTrigger.get(i);
			ret = CAEN_DGTZ_SetDPPPreTriggerSize(m_boardHandle, i, ptgrSize);
			if (ret) {
				cerr << "Error CAEN_DGTZ_SetDPPPreTriggerSize" << endl;
				close();
				return;
			}

			// Set polarity
			CAEN_DGTZ_PulsePolarity_t polarity = (CAEN_DGTZ_PulsePolarity_t)owner()->pulsePolarity.get(i);
			ret = CAEN_DGTZ_SetChannelPulsePolarity(m_boardHandle, i, polarity);
			if (ret) {
				cerr << "Error CAEN_DGTZ_SetChannelPulsePolarity" << endl;
				close();
				return;
			}
		}
	}

	close();
}

/*!
 * \brief Set acquisition mode command implementation
 */
void RealCAENPsdDriver::setAcqMode() {

	open();
	boardParams.DPPAcqMode = (CAEN_DGTZ_DPP_AcqMode_t)owner()->dppAcqMode();
	boardParams.SaveMode = (CAEN_DGTZ_DPP_SaveParam_t)owner()->saveMode();
	ret = CAEN_DGTZ_SetDPPAcquisitionMode(m_boardHandle, boardParams.DPPAcqMode, boardParams.SaveMode);
	if (ret) {
		cerr << "Error CAEN_DGTZ_SetDPPAcquisitionMode: " << ret << endl;
	}
	close();
}

/*!
 * \brief Read Infos command implementation
 */
void RealCAENPsdDriver::readInfos() {

	open();
	ret = CAEN_DGTZ_GetInfo(m_boardHandle, &boardInfo);
	close();

	owner()->nbBits = boardInfo.ADC_NBits;
	owner()->model = boardInfo.Model;
	owner()->channels = boardInfo.Channels;
	owner()->serialNumber = boardInfo.SerialNumber;
	owner()->modelName = boardInfo.ModelName;
	owner()->rocFirmware = boardInfo.ROC_FirmwareRel;
	owner()->amcFirmware = boardInfo.AMC_FirmwareRel;
}

}
